@@ -20,17 +20,20 @@ from iwf.iwf_api.api.default import (
     post_api_v1_workflow_rpc,
     post_api_v1_workflow_search,
     post_api_v1_workflow_searchattributes_get,
+    post_api_v1_workflow_searchattributes_set,
     post_api_v1_workflow_signal,
     post_api_v1_workflow_start,
     post_api_v1_workflow_stop,
     post_api_v1_workflow_timer_skip,
     post_api_v_1_workflow_get_with_wait,
+    post_api_v_1_workflow_wait_for_state_completion,
 )
 from iwf.iwf_api.models import (
     EncodedObject,
     ErrorResponse,
     IDReusePolicy,
     PersistenceLoadingPolicy,
+    SearchAttribute,
     SearchAttributeKeyAndType,
     WorkflowConfig,
     WorkflowGetDataObjectsRequest,
@@ -53,6 +56,8 @@ from iwf.iwf_api.models import (
     WorkflowStopRequest,
     WorkflowAlreadyStartedOptions,
     KeyValue,
+    WorkflowSetSearchAttributesRequest,
+    WorkflowWaitForStateCompletionRequest,
 )
 from iwf.iwf_api.types import Response
 from iwf.reset_workflow_type_and_options import ResetWorkflowTypeAndOptions
@@ -69,6 +74,8 @@ class UnregisteredWorkflowOptions:
     workflow_config_override: Optional[WorkflowConfig] = None
     workflow_already_started_options: Optional[WorkflowAlreadyStartedOptions] = None
     initial_data_attributes: Optional[dict[str, Any]] = None
+    wait_for_completion_state_execution_ids: Optional[list[str]] = None
+    wait_for_completion_state_ids: Optional[list[str]] = None
 
 
 T = TypeVar("T")
@@ -160,6 +167,16 @@ class UnregisteredClient:
 
             request.workflow_start_options = start_options
 
+            if options.wait_for_completion_state_execution_ids:
+                request.wait_for_completion_state_execution_ids = (
+                    options.wait_for_completion_state_execution_ids
+                )
+
+            if options.wait_for_completion_state_ids:
+                request.wait_for_completion_state_ids = (
+                    options.wait_for_completion_state_ids
+                )
+
         response = post_api_v1_workflow_start.sync_detailed(
             client=self.api_client,
             json_body=request,
@@ -231,6 +248,7 @@ class UnregisteredClient:
         data_attribute_policy: Optional[PersistenceLoadingPolicy],
         all_defined_search_attribute_types: list[SearchAttributeKeyAndType],
         return_type_hint: Optional[Type[T]] = None,
+        search_attribute_policy: Optional[PersistenceLoadingPolicy] = None,
     ) -> Optional[T]:
         request = WorkflowRpcRequest(
             input_=self.client_options.object_encoder.encode(input),
@@ -242,6 +260,8 @@ class UnregisteredClient:
         )
         if data_attribute_policy is not None:
             request.data_attributes_loading_policy = data_attribute_policy
+        if search_attribute_policy is not None:
+            request.search_attributes_loading_policy = search_attribute_policy
 
         try:
             response = post_api_v1_workflow_rpc.sync_detailed(
@@ -426,6 +446,56 @@ class UnregisteredClient:
         )
         return handler_error_and_return(response)
 
+    def set_workflow_search_attributes(
+        self,
+        workflow_id: str,
+        workflow_run_id: str,
+        search_attributes: list[SearchAttribute],
+    ):
+        request = WorkflowSetSearchAttributesRequest(
+            workflow_id=workflow_id,
+            workflow_run_id=workflow_run_id,
+            search_attributes=search_attributes,
+        )
+        response = post_api_v1_workflow_searchattributes_set.sync_detailed(
+            client=self.api_client,
+            json_body=request,
+        )
+        return handler_error_and_return(response)
+
+    def wait_for_state_execution_completion_with_state_execution_id(
+        self, workflow_id: str, state_execution_id: str
+    ):
+        request = WorkflowWaitForStateCompletionRequest(
+            workflow_id=workflow_id,
+            state_execution_id=state_execution_id,
+            wait_time_seconds=self.client_options.long_poll_api_max_wait_time_seconds,
+        )
+
+        response = post_api_v_1_workflow_wait_for_state_completion.sync_detailed(
+            client=self.api_client,
+            json_body=request,
+        )
+
+        return handler_error_and_return(response)
+
+    def wait_for_state_execution_completion_with_wait_for_key(
+        self, workflow_id: str, state_id: str, wait_for_key: str
+    ):
+        request = WorkflowWaitForStateCompletionRequest(
+            workflow_id=workflow_id,
+            state_id=state_id,
+            wait_for_key=wait_for_key,
+            wait_time_seconds=self.client_options.long_poll_api_max_wait_time_seconds,
+        )
+
+        response = post_api_v_1_workflow_wait_for_state_completion.sync_detailed(
+            client=self.api_client,
+            json_body=request,
+        )
+
+        return handler_error_and_return(response)
+
 
 def handler_error_and_return(response: Response):
     if response.status_code != http.HTTPStatus.OK:
